#include "fileparser.h"
#include <QDebug>
#include <QStringList>
#include <QFile>
#include <iostream>
#include <fstream>
#include <string>

/*!
* read opens a file for the given filename and exports the containing data line by line into the list of the type TYPE passed as an argument
* \param filename The filename in a QString representation
* \param list The data structure to be written into. Several types of list are allowed.
* \return boolean value that indicates the success of the file reading process.
*/
template<typename TYPE>
bool FileParser::read(QString filename, TYPE &list) {
	if(filename == NULL) {
		qDebug() << "No filename entered";
		return false;
	}

	QFile file(filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;

    QTextStream in(&file);

	while(!in.atEnd()) { 
		QString line = in.readLine();
		if(!parseFromString(list, line))
			qDebug() << line;
	}

	file.close();
	return true;
}
//explicit declaration of possible template types
template bool FileParser::read<QList<Airport*>>(QString filename, QList<Airport*> &list);
template bool FileParser::read<QList<Route*>>(QString filename, QList<Route*> &list);
template bool FileParser::read<QMap<QString, int>>(QString filename, QMap<QString, int> &map);
template bool FileParser::read<QMap<int, int>>(QString filename, QMap<int, int> &map);


/*!
* Parsing a single line from the instream into a List of objects of type <Airport>
* \param list QList of Airport objects. The data structure to be written into.
* \param line The line to be parsed.
* \return boolean value that indicates the success of the file reading process.
*/
bool FileParser::parseFromString(QList<Airport*> &list, QString line) {
	line.remove(QChar('"'));

	QStringList splitString = line.split(",");	

	bool ok;
	if(splitString.size() != 11) { 
		qDebug() << "Wrong number of arguments in line";
		return false;
	}

	int id			= splitString[0].toInt(&ok);
	QString name	= splitString[1];
	QString city	= splitString[2];
	QString country = splitString[3];
	QString iata	= splitString[4];
	QString icao	= splitString[5];
	double lat		= splitString[6].toDouble(&ok);
	double lon		= splitString[7].toDouble(&ok);
	double alt		= splitString[8].toDouble(&ok);
	double timez	= splitString[9].toDouble(&ok);
	QString dst		= splitString[10];

	if(!ok) { 
		qDebug() << "Something wrong in argument conversion";
		return false;
	}
	
	list.append(new Airport(id, name, city, country, iata, icao, lat, lon, alt, timez, dst));

	//if(list.size() % 100 == 0) //for debugging purposes
	//	qDebug() << list.last()->toString();

	return true;
}

/*!
* Parsing a single line from the instream into a List of objects of type <Route>
* \param list QList of Route objects. The data structure to be written into.
* \param line The line to be parsed.
* \return boolean value that indicates the success of the file reading process.
*/
bool FileParser::parseFromString(QList<Route*> &list, QString line) {
	line.remove(QChar('"'));

	QStringList splitString = line.split(",");	

	bool ok;
	if(splitString.size() != 9) { 
		qDebug() << "Wrong number of arguments in line";
		return false;
	}

	QString airline		= splitString[0];
	int airline_ID		= splitString[1].toInt(&ok);
	QString origin		= splitString[2];
	int origin_ID		= splitString[3].toInt(&ok);
	QString dest		= splitString[4];
	int dest_ID			= splitString[5].toInt(&ok);
	QString codeshare	= splitString[6];
	int stops			= splitString[7].toInt(&ok);
	QString equipment	= splitString[8];


	if(!ok) { 
		qDebug() << "Something wrong in argument conversion";
		return false;
	}

	list.append(new Route(airline, airline_ID, origin, origin_ID, dest, dest_ID, codeshare, stops, equipment));
	
	//if(list.size() % 100 == 0) //for debugging purposes
	//	qDebug() << list.last()->toString();
	
	return true;
}

/*!
* Parsing a single line from the instream into a QMap objects of type <QString, int>
* \param list QMap of objects. The data structure to be written into.
* \param line The line to be parsed.
* \return boolean value that indicates the success of the file reading process.
*/
bool FileParser::parseFromString(QMap<QString, int> &map, QString line) {
	QStringList splitString = line.split(" ");	
	bool ok;

	int value		= splitString[0].toInt(&ok);
	QString key		= splitString[1];
	for(int i = 2; i < splitString.size(); i++)
		 key += " " + splitString[i];

	if(!ok) { 
		qDebug() << "Something wrong in argument conversion";
		return false;
	}
	
	map.insertMulti(key, value);
	return true;

}

/*!
* Parsing a single line from the instream into a QMap objects of type <int, int>
* \param list QMap of objects. The data structure to be written into.
* \param line The line to be parsed.
* \return boolean value that indicates the success of the file reading process.
*/
bool FileParser::parseFromString(QMap<int, int> &map, QString line) {
	QStringList splitString = line.split(" ");	
	bool ok;

	int key 		= splitString[0].toInt(&ok);
	int value		= splitString[1].toInt(&ok);
		
	if(!ok) { 
		qDebug() << "Something wrong in argument conversion";
		return false;
	}

	map.insertMulti(key, value);
	
	return true;
}