#include "forcebundler.h"
#include <QDebug>

/* calculate pairwise edge compatibilities */
QVector<QVector<double>>* ForceBundler::calculateCompatibilities(QList<Edge*>* edges) {
	int numEdges = edges->count();
	QVector<QVector<double>>* compatibilities = new QVector<QVector<double>>(numEdges, QVector<double>(numEdges, 0.0));
	//QVector<QVector<double>>* compatibilities = new QVector<QVector<double>>(numEdges, QVector<double>(numEdges, 0.0));
	for(int iEdge1 = 0; iEdge1 < numEdges; iEdge1++) {
		Edge* edge1 = (*edges)[iEdge1];
		for(int iEdge2 = 0; iEdge2 < numEdges; iEdge2++) {
			if(iEdge1 == iEdge2 || (*compatibilities)[iEdge1][iEdge2] != 0.0) continue;
			Edge* edge2 = (*edges)[iEdge2];
			double compatibility = edge1->compatibility(edge2);
			(*compatibilities)[iEdge1][iEdge2] = compatibility;
			(*compatibilities)[iEdge2][iEdge1] = compatibility;
		}
	}
	return compatibilities;
}

/* calculate pairwise edge compatibilities */
QVector<QMap<int, double>>* ForceBundler::compatibleEdges(QList<Edge*>* edges) {
	int numEdges = edges->count();
	QVector<QMap<int, double>>* compatibilities = new QVector<QMap<int, double>>(numEdges, QMap<int, double>());
	for(int iEdge1 = 0; iEdge1 < numEdges; iEdge1++) {
		Edge* edge1 = (*edges)[iEdge1];
		for(int iEdge2 = iEdge1+1; iEdge2 < numEdges; iEdge2++) {
			Edge* edge2 = (*edges)[iEdge2];
			double compatibility = edge1->compatibility(edge2);
			if(compatibility > 0.1) {
				(*compatibilities)[iEdge1].insert(iEdge2, compatibility);
				(*compatibilities)[iEdge2].insert(iEdge1, compatibility);
			}
		}
	}
	return compatibilities;
}

void ForceBundler::bundleEdges(int cycles, int iterations, double springConstant) {
	double stepSize = 0.01;
	double progress = 0;
	double progressStep = 100.0 / (cycles * iterations);

	QList<Edge*> edges = *(graph->getEdges());
	int numEdges = edges.count();
	int numSubdivisions = 1;
	QVector<QMap<int, double>>* compatibilities = compatibleEdges(&edges);
	/** DO ALL THE CYCLES! **/
	for(int cycle = 0; cycle < cycles; cycle++) { //paper suggests: cycles = 6
		
		//increase number of subdivisions
		for(Edge* edge: edges) {
			// edge->increaseSubdivisions(); //naive increasing
			edge->increaseSubdivisions(numSubdivisions); //increasing as suggested in paper
		}
		numSubdivisions *= 2;

		/** DO ALL THE ITERATIONS! **/
		for(int iter = 0; iter < iterations; iter++) { //paper suggests: iterations_0 = 50
			/** DO ALL THE EDGES! **/
			for(int iEdge = 0; iEdge < numEdges; iEdge++) {
				Edge* edge = edges[iEdge];
		
				QPolygonF subdivisionPoints = edge->getSubdivisionPoints();
				QPolygonF newSubdivisionPoints;
				newSubdivisionPoints.append(edge->start()->toPointF());

				int numSubdivisionPoints = subdivisionPoints.count();
				int numSegments = numSubdivisionPoints - 1;

				double length = edge->length();
				double k_p = springConstant / (length * numSegments);

				//traverse all subdivision points
				for(int pos = 1; pos < numSegments; pos++) {
					QVector2D point_left	= QVector2D(subdivisionPoints[pos-1]);
					QVector2D point			= QVector2D(subdivisionPoints[pos]);
					QVector2D point_right	= QVector2D(subdivisionPoints[pos+1]);
					
					// Calculate spring force
					QVector2D force_spring = k_p * ((point_left - point) + (point_right - point));
					QVector2D force_electrostatic = QVector2D();

					/** COMPARE ALL THE EDGES! **/
					
					//BELOW: SIMPLY GO THROUGH ALL THE EDGES NAIVELY
					//for(int jEdge = 0; jEdge < numEdges; jEdge++) {
					//	if(iEdge == jEdge) continue;
					//	
					//	double compatibility = (*compatibilities)[iEdge][jEdge];
					//	if(compatibility <= 0.1) continue;

					//GO THROUGH ONLY COMPATIBLE EDGES - MORE EFFICIENT
					QMap<int, double> compatibleEdges = (*compatibilities)[iEdge];
					QMapIterator<int, double> i(compatibleEdges);
					while (i.hasNext()) {
						i.next();
						Edge* other = edges[i.key()];
						double compatibility = i.value();
						
						//find out edge orientation
						double similar		= qMin(edge->start()->distanceToPoint(*other->start()), 
												   edge->end()->distanceToPoint(*other->end()));
						double different	= qMin(edge->start()->distanceToPoint(*other->end()), 
											       edge->end()->distanceToPoint(*other->start()));

						//compare this subdivision point with the respective subdivision point from all other compatible edges
						QVector2D point_other;
						if(similar < different) point_other = QVector2D(other->getSubdivisionPoints()[pos]);
						else                    point_other = QVector2D(other->getSubdivisionPoints()[numSegments - pos]);

						QVector2D forceVector = (point_other - point).normalized();
						float dist = forceVector.length();
						if(dist == 0) continue;

						force_electrostatic += forceVector * compatibility;
					}
					QPointF newPoint = point.toPointF() + stepSize * (force_electrostatic + force_spring).toPointF();
					if(!QRectF(QPointF(0,0), QPointF(1,1)).contains(newPoint)) {
						qDebug() << newPoint;
						newSubdivisionPoints.append(point.toPointF());
					} else { 
						newSubdivisionPoints.append(newPoint);
					}
				}
				newSubdivisionPoints.append(QPointF(edge->end()->toPointF()));
				edge->setNewSubdivisionPoints(newSubdivisionPoints);
			}
			//update all the subdivision polygons
			for(Edge* edge: edges)
				edge->updateSubdivisionPoints();

			int oldProgress = qRound(progress);
			int newProgress = qRound(progress += progressStep);
			if(newProgress > oldProgress)
				emit setProgress(1.0);
			
			//renderWidget->refresh();
		}
		// Decrease iterations and stepSize for next cycle
		iterations = qRound((double)iterations*(2/3));
		stepSize = stepSize / 2.0;
	}
	delete compatibilities;
}