#include "graph.h"
#include <QDebug>
/*!
* Increase the edge's subdivision points by always naively inserting additional subdivisions between existing ones: 1 -> 3 -> 7 -> ...
* <p>This approach does not adhere to the paper's suggestion of increasing the nodes in this succession: 1 -> 2 -> 4 -> 8 -> 16 -> 32
*/
void Edge::increaseSubdivisions(void) { 
	QPolygonF polygon;
	polygon << subdivisionPoints[0];
	for(int i=1; i < subdivisionPoints.size(); i++) {
		polygon << Graph::midPoint(subdivisionPoints[i-1], subdivisionPoints[i]);
		polygon << subdivisionPoints[i];
	}
	subdivisionPoints = polygon;
}

/*!
* divide the edge's total length and then convert to a specific number of subdivisions inserting the new subdivision points in equidistant intervals.
* <p>This function can adhere to the succession 1 -> 2 -> 4 -> 8 -> 16 -> 32 suggested in the paper but is more complex than the overloaded increaseSubdivisions() taking no arguments.
* \param numSubdivisions number of subdivisions the edge should be divided into
**/
void Edge::increaseSubdivisions(int numSubdivisions) { 
	QPolygonF polygon;

	double currentEdgeLength = 0.0;
    for (int i = 1; i < subdivisionPoints.size(); i++)
		currentEdgeLength += QVector2D(subdivisionPoints[i - 1]).distanceToPoint(QVector2D(subdivisionPoints[i]));
        
    double newSectionLength = currentEdgeLength / (numSubdivisions + 1);
    double distanceToNextPoint = newSectionLength;

	polygon << start()->toPointF(); //insert start point to polygon

    for (int i = 1; i < subdivisionPoints.size(); i++) {
		QPointF p = subdivisionPoints[i];
		QPointF p_prev = subdivisionPoints[i - 1];

		double currentSectionLength = QVector2D(p_prev).distanceToPoint(QVector2D(p));
        double remainingSectionLength = currentSectionLength;

        while (distanceToNextPoint < remainingSectionLength) {
            remainingSectionLength -= distanceToNextPoint;
            polygon << QPointF(p_prev + (1.0 - (remainingSectionLength / currentSectionLength)) * (p - p_prev));
			distanceToNextPoint = newSectionLength;
        }
        distanceToNextPoint -= remainingSectionLength;
    }
	if(polygon.count() < numSubdivisions + 2) polygon << end()->toPointF(); //insert end point to polygon

	subdivisionPoints = polygon;
}

/*!
* calculate the edge's compatibility by multiplying the individual compatibility parameters.
* \param other Edge to be compared with this one.
*/
double Edge::compatibility(Edge* other) {
	double angular = compatibility_angular(other);
	double scale = compatibility_scale(other);
	double positional = compatibility_positional(other);
	double visibility = compatibility_visibility(other);

	return angular * scale * positional	* visibility;
}

/*!
* calculate the angular compatibility between this edge and the one passed in the function.
* <p>paper suggests calculating the cosine and then the arcus cosine which I left out because I didn't see why?
* \param other Edge to be compared with this one.*/
//triplechecked =)
double Edge::compatibility_angular(Edge* other) {
	double compatibility = acos(QVector2D::dotProduct(vector(), other->vector()) / (length() * other->length()));
	compatibility = abs(cos(compatibility));
	if(compatibility < 0 || compatibility > 1) {
		qDebug() << "error in angular compatibility!";
		compatibility = 0.0;// = qMin(1.0, qMax(0.0, compatibility));
	}
	return compatibility;
}

/*!
* calculate the compatibility between the scale of this edge and the one passed in the function
* \param other Edge to be compared with this one.
*/
//triplechecked =)
double Edge::compatibility_scale(Edge* other){
	double averageLength = (length() + other->length()) / 2.0;

	double compatibility = 2.0 / ((averageLength / qMin(length(), other->length())) + (qMax(length(), other->length()) / averageLength));
	if(compatibility < 0 || compatibility > 1) {
		qDebug() << "error in scale compatibility!";
		compatibility = qMin(1.0, qMax(0.0, compatibility));
	}
	return compatibility;
}

/*!
* calculate the compatibility between the spatial distance of this edge and the one passed in the function
* \param other Edge to be compared with this one.
*/
//triplechecked =)
double Edge::compatibility_positional(Edge* other) {
	double averageLength = (length() + other->length()) / 2.0;
	
	double compatibility = averageLength / (averageLength + midPoint().distanceToPoint(other->midPoint()));
	if(compatibility < 0 || compatibility > 1) {
		qDebug() << "error in positional compatibility!";
		compatibility = qMin(1.0, qMax(0.0, compatibility));
	}
	return compatibility;
}

/*!
* calculate the compatibility between the visibility of this edge and the one passed in the function
* visibilities are calculated in both direction, using the minimal value of both results.
* \param other Edge to be compared with this one.
*/
//doublechecked
double Edge::compatibility_visibility(Edge* other){
	double visible1 = visibilityFactor(other);
	double visible2 = other->visibilityFactor(this);
	double compatibility = qMin(visible1, visible2);

	if(compatibility < 0 || compatibility > 1) {
		qDebug() << "error in visibility compatibility!";
		compatibility = qMin(1.0, qMax(0.0, compatibility));
	}
	return compatibility;
}


/*!
* Evaluate the factor of visibility from one edge to another according to the algorithm proposed in the paper.
* \param other Edge to be compared with this one.
*/
double Edge::visibilityFactor(Edge* other) {
    QPair<QVector2D,QVector2D> intersections = bandOfSight(other);
	QVector2D i_0 = intersections.first;
	QVector2D i_1 = intersections.second;
	
	QVector2D i_m = Graph::midPoint(i_0, i_1);
    QVector2D p_m = Graph::midPoint(*start(), *end());
    return qMax(0.0, 1.0 - 2.0 * p_m.distanceToPoint(i_m) / (i_0.distanceToPoint(i_1)));
}

/*!
* Calculate the two points composing the band of sight according to the algorithm proposed in the paper.
* \param other Edge to be compared with this one.
*/
QPair<QVector2D,QVector2D> Edge::bandOfSight(Edge* other) {
	QVector2D p_0 = *start();
	QVector2D p_1 = *end();
	QVector2D q_0 = *other->start();
    QVector2D q_1 = *other->end();

	double dist_1 = ((p_0.y() - q_0.y()) * (p_0.y() - p_1.y()) - (p_0.x() - q_0.x()) * (p_1.x() - p_0.x())) / vector().lengthSquared();
	double dist_2 = ((p_0.y() - q_1.y()) * (p_0.y() - p_1.y()) - (p_0.x() - q_1.x()) * (p_1.x() - p_0.x())) / vector().lengthSquared();

	QVector2D i_0 = p_0 + dist_1 * (p_1 - p_0);
	QVector2D i_1 = p_0 + dist_2 * (p_1 - p_0);

    return QPair<QVector2D,QVector2D>(i_0, i_1);
}