#pragma once
#include <QList>
#include <QPoint>
#include <QString>
#include <QVector2D>
#include <QPolygon>

/**
 * Datastructure for one Node instance
 */
class Node: public QVector2D {
protected: 
	int		degree;
	int		group;
	QString description;
public:
	/**
	 * Constructor for a node, initializes a node with position (0, 0)
	 */
	Node(): QVector2D(), degree(0), group(-1) {};
	/**
	 * Constructor for a node, initializes a node with position (x, y)
	 * \param x x-coordinate for Node's position
	 * \param y y-coordinate for Node's position
	 */
	Node(double x, double y): QVector2D(x, y), degree(0), group(-1) {};
	~Node(void) {};
	operator QVector2D()			{return QVector2D(x(), y());};
	void	set(double x, double y)	{setX(x); setY(y);};
	void	set(QPointF p)			{setX(p.x()); setY(p.y());};
	void	set(QVector2D p)		{setX(p.x()); setY(p.y());};

	/** UNUSED
	 * setter for node affiliation with a new group g. 
	 * \param g new group that node belongs to
	 */
	void	setGroup(int g)			{group = g;};
	/** UNUSED
	 * getter for node affiliation with a group. 
	 * \return g group that node belongs to
	 */
	int		getGroup(void)			{return group;};

	/** increment node's degree by 1. */
	void	increment(void)			{degree++;};
	/** \return the node's degree */
	int		getDegree(void)			{return degree;};
	/** set the node's descriptive string \param s new info string for the node */
	void	setInfo(QString s)		{description = s;};
	/** return the node's descriptive string \return description */
	QString	info()					{return description;};
	QString toString()				{return QString("[") += QString::number(x()) += QString(",") += QString::number(y())+= QString("]");};
};

/**
 * Datastructure for one Edge instance
 */
class Edge {
protected:
	Node* node_start;
	Node* node_end;
	QPolygonF subdivisionPoints;
	QPolygonF newSubdivisionPoints;

public:
	Edge() {};
	/**
	 * Constructor for an edge, initializes the edges with a start and end node and an empty list of subdivision points
	 * \param n1 pointer to start node
	 * \param n2 pointer to end node
	 */
	Edge(Node* n1, Node* n2) {
		node_start	= n1;
		node_end	= n2;
		subdivisionPoints << n1->toPointF() << n2->toPointF();
	};

	~Edge(void){};

	/**
	 * Store a list of subdivision points temporarily, while the bundling is in progress.
	 * \param p new polygon of subdivision points
	 */
	void setNewSubdivisionPoints(QPolygonF p){newSubdivisionPoints = p;};
	/** After the bundling is finished, update the list of the edge's subdivision points to the previously stored new ones. s*/
	void updateSubdivisionPoints(){subdivisionPoints = newSubdivisionPoints; newSubdivisionPoints = QPolygonF();};

	/** Return the polygon containing the edge's subdivision points. \return subdivision polygon. */
	QPolygonF	getSubdivisionPoints()					{return subdivisionPoints;};
	void		clearSubdivisionPoints()				{subdivisionPoints.clear();};
	
	Node*		start(void)								{return node_start;};
	Node*		end(void)								{return node_end;};
	QVector2D	midPoint(void)							{return (*start() + *end())/2.0;};
	QVector2D	vector(void)							{return *end() - *start();};
	double		length(void)							{return vector().length();};

	void		increaseSubdivisions(void);
	void		increaseSubdivisions(int numSubdivisions);
	double		compatibility(Edge* other);
	QString		toString(void) {return this->start()->toString() + " -> " + this->end()->toString();};
protected:
	/* COMPATIBILITY CALCULATION FUNCTIONS */
	double		compatibility_angular(Edge* other);	
	double		compatibility_scale(Edge* other);
	double		compatibility_positional(Edge* other);
	double		compatibility_visibility(Edge* other);
	double		visibilityFactor(Edge* other);
	QPair<QVector2D,QVector2D> bandOfSight(Edge* other);
};

/*
 * Datastructure for one Graph instance
**/
class Graph {
public:
	Graph(void) {};
	~Graph(void) {
		qDeleteAll(edges.begin(), edges.end());
		edges.clear();

		qDeleteAll(nodes.begin(), nodes.end());
		nodes.clear();
	};
	void clear(void)	{
		for (int i=0; i<edges.count(); i++)
			delete (edges.takeAt(0));
		edges.clear();

		for (int i=0; i<nodes.count(); i++)
			delete (nodes.takeAt(0));
		nodes.clear();
	}

	/** Return the list of all the graph's nodes. \return QList of pointers to all the nodes */
	QList<Node*>*	getNodes()			{return &nodes;};
	/** Return the list of all the graph's edges. \return QList of pointers to all the edges */
	QList<Edge*>*	getEdges()			{return &edges;};
	/**Add a new node to the graph's list of nodes \param n new node */
	void			addNode(Node* n)	{nodes<<n;};
	/**Add a new edge to the graph's list of edges \param e new edge */
	void			addEdge(Edge* e)	{edges<<e;};

protected: 
	QList<Node*>	nodes;
	QList<Edge*>	edges;

public:
	/** static convenience function returning the mean point between two QVector2D objects \param a first QVector2D \param b second QVector2D */
	static QVector2D midPoint(QVector2D a, QVector2D b) {return (a + b) / 2.0;};
	/** static convenience function returning the mean point between two QPointF objects \param a first QPointF \param b second QPointF */
	static QPointF   midPoint(QPointF a, QPointF b)		{return (a + b) / 2.0;};
};