#include "renderthread.h"
#include <math.h>
#include <QtWidgets>


RenderThread::RenderThread(QObject *parent): QThread(parent) {
    restart = false;
    abort = false;

}

RenderThread::~RenderThread() {
    mutex.lock();
    abort = true;
    condition.wakeOne();
    mutex.unlock();

    wait();
}

void RenderThread::render(Graph* g, QSize resSize, QPen ePen, QBrush nBrush, QPen nPen, double nRad, bool vEdges, bool vNodes, bool vDescr, bool w, bool b) {
    QMutexLocker locker(&mutex);

	this->graph = g;
    this->resultSize = resSize;
	this->edgePen = ePen;
	this->nodeBrush = nBrush;
	this->nodePen = nPen;
	this->nodeRadius = nRad;
	this->visibleEdges = vEdges;
	this->visibleNodes = vNodes;
	this->showDescriptions = vDescr;
	this->wrap = w;
	this->bundle = b;

    if (!isRunning()) {
        start(LowPriority);
    } else {
        restart = true;
        condition.wakeOne();
    }
}

void RenderThread::run() {
    forever {
        mutex.lock();
			QSize resultSize = this->resultSize;
			/* TEMP VARIABLES */
			bool visibleEdges = this->visibleEdges;
			bool visibleNodes = this->visibleNodes;
			bool wrap = this->wrap;
			bool bundle = this->bundle;
		
			QPen edgePen = this->edgePen;
			QBrush nodeBrush = this->nodeBrush;
			QPen nodePen = this->nodePen;
			double nodeRadius = this->nodeRadius;
        mutex.unlock();

		QImage image(resultSize, QImage::Format_ARGB32_Premultiplied);
		image.fill(Qt::transparent);

		/* START MY PART */
		QPainter painter;
		painter.begin(&image);
		painter.setBackgroundMode(Qt::TransparentMode);
		painter.setRenderHint(QPainter::Antialiasing);
		painter.setPen(edgePen);
			int numEdges = 0, numNodes = 0;

			if(visibleEdges) { /*draw edges*/
				QList<Edge*>* edges = graph->getEdges();
				numEdges = edges->count();

				for(int iEdge = 0; iEdge < numEdges; iEdge++) {
					if (restart)
						break;
					if (abort)
						return;
					Edge* edge		= edges->at(iEdge);

					QPointF start	= QPointF(getWindowCoordinate(edge->start()->toPointF()));
					QPointF end		= QPointF(getWindowCoordinate(edge->end()->toPointF()));
			
					/*if wrapEdges: draw edges between points close to left and right border as though wrapping around earth
					* _______________             _______________
					* |             |             |             |
					* |             | instead of  |             | 
					* |-*       *---|             | *-------*   |
					* |_____________|             |_____________|
					**/
					if(!bundle && wrap && qAbs(start.x() - end.x()) > resultSize.width() / 2) { //do wrap for edges with greater distance than half of window width
				
						QPointF left, right; //determine which point is left and which is right
						left	= (start.x() < end.x() ? start : end);
						right	= (left == start	   ? end : start);

						//calculate percentage of total line going from left point to border: divide left coordinate by total length of line
						double leftPercent = left.x() / (left.x() + (resultSize.width() - right.x())); 

						//determine y coordinate of intersection by interpolating between y values according to above percentage
						double yInterp = left.y() - leftPercent * (left.y() - right.y());

						painter.drawLine(left, QPointF(0.0, yInterp));
						painter.drawLine(right, QPointF(resultSize.width(), yInterp));

					} else {
						QPolygonF poly = edge->getSubdivisionPoints();
						if(poly.empty() || bundle == false)
							painter.drawLine(start, end);
						else
							painter.drawPolyline(getWindowCoordinates(poly));
					}

					emit setStatus("Rendering graph | Drawing edges: " + QString::number((iEdge * 100) / numEdges) + "%");
				}
			}

			painter.setBrush(nodeBrush);
			painter.setPen(nodePen);

			if(visibleNodes) { /*draw nodes*/
				QList<Node*>* nodes = graph->getNodes();
				numNodes = nodes->count();
		
				for(int iNode = 0; iNode < numNodes; iNode++) {
					if (restart)
						break;
					if (abort)
						return;
					QPointF position = getWindowCoordinate(nodes->at(iNode)->toPointF());
			
					double radius = qMax(log(nodes->at(iNode)->getDegree()), nodeRadius);
					painter.drawEllipse(position, radius, radius);
					if(showDescriptions)
						painter.drawText(position + QPointF(-8.0, 15.0), nodes->at(iNode)->info());
			
					emit setStatus("Rendering graph | Drawing nodes: " + QString::number((iNode * 100) / numNodes) + "%");
				}
			}
		painter.end();
		/* END MY PART */
        if (!restart) {
            emit renderedImage(image);

			QString message;
			message += visibleNodes ? QString::number(numNodes) + " Nodes" : "";
			message += visibleNodes&&visibleEdges ? " | " : "";
			message += visibleEdges ? QString::number(numEdges) + " Edges" : "";
			emit setStatus(message);
		}

        mutex.lock();
        if (!restart)
            condition.wait(&mutex);
        restart = false;
        mutex.unlock();
    }
}