#ifndef RENDERTHREAD_H
#define RENDERTHREAD_H

#include "graph.h"
#include <QMutex>
#include <QSize>
#include <QThread>
#include <QPen>
#include <QBrush>
#include <QWaitCondition>

class QImage;

/*! 
 * RenderThread subclasses QThread to perform the graph rendering in a different thread from the UI.
 * <p> This prevents the UI from freezing while the rendering process is going on which is particularly essential for big graphs.
 */
class RenderThread : public QThread {
    Q_OBJECT

public:
	/*! 
	 * Constructor for a new RenderThread
	 */
    RenderThread(QObject *parent = 0);
    /*! 
	 * Destructor for the RenderThread
	 */
	~RenderThread();

	/*! 
	 * function call to start the rendering of the thread. The function is passed all the necessary arguments, which it copies to local variables 
	 * \param g Graph to be rendered
	 * \param resSize size of the current viewport
	 * \param ePen QPen containing the properties for edge drawing
	 * \param nBrush QBrush containing the properties for the node fill
	 * \param nPen QPen containing the properties for the node outline
	 * \param nRad QPen containing the properties for the node outline
	 * \param vEdges bool determining the visibility of the edges
	 * \param vNodes bool determining the visibility of the nodes
	 * \param w bool determining whether the edges are allowed to wrap
	 * \param b bool determining whether the rendering should draw bundled edges
	 */
    void render(Graph* g, QSize resSize, QPen ePen, QBrush nBrush, QPen nPen, double nRad, bool vEdges = true, bool vNodes = true,  bool vDescr = true, bool w = false, bool b = false);

signals:
	/*! emits a rendered image when the render thread has finished */
    void renderedImage(const QImage &image);
	/*! signal to notify the main window's status bar of the progress of graph generation */
	void setStatus(QString s);

protected:
	/*! main thread method */
    void run();

	Graph* graph;
    QMutex mutex;
    QWaitCondition condition;
    QSize resultSize;
    bool restart;
    bool abort;

	/* GRAPH VARIABLES */
	bool visibleEdges;
	bool visibleNodes;
	bool showDescriptions;
	bool wrap;
	bool bundle;
		
	QPen edgePen;
	QBrush nodeBrush;
	QPen nodePen;
	double nodeRadius;

	/*! convert a normalized graph coordinate to window coordinate \param point QPointF in normalized coordinates \return the converted coordinate */
    QPointF getWindowCoordinate(QPointF point) {return QPointF(point.x() * resultSize.width(), point.y() * resultSize.height());}
	/*! converts normalized graph coordinates to window coordinates for the entire polygon \param polygon QPolygonF containing normalized coordinates \return polygon with converted coordinates */
	QPolygonF getWindowCoordinates(QPolygonF poly) {QMatrix transform; transform.scale(resultSize.width(), resultSize.height()); return transform.map(poly);};
};

#endif // RENDERTHREAD_H