#include "renderwidget.h"
#include <math.h>
#include <QPainter>
#include <QKeyEvent>
#include <QSvgRenderer>

RenderWidget::RenderWidget(QWidget *parent, Graph* g) : QWidget(parent), graph(g) {
    connect(&thread, SIGNAL(renderedImage(QImage)), this, SLOT(updatePixmap(QImage)));
	connect(&thread, SIGNAL(setStatus(QString)), parent, SLOT(updateStatus(QString)));
	
#ifndef QT_NO_CURSOR
    setCursor(Qt::CrossCursor);
#endif
}

void RenderWidget::useBackgroundImage(QString path) {
	useBackground(true);
	backgroundImagePath = path;

	if(path.endsWith("svg")) {
		QSvgRenderer svg(path);
		QImage img(width(), height(), QImage::Format_ARGB32_Premultiplied);
		img.fill(Qt::white);
		QPainter painter(&img);
		svg.render(&painter);
		backgroundImage = QPixmap::fromImage(img); 
	} else if(path.endsWith("png") || path.endsWith("jpg")) {
		backgroundImage = QPixmap(path);
	} else {
		backgroundImagePath = "";
		useBackground(false);
	}
}

void RenderWidget::useBackgroundColor(QColor color) {
	backgroundSolid = QBrush(color);
	useBackground(false);
}

void RenderWidget::setNodeProperties(double radius, QColor fillColor, QColor lineColor, double lineWidth) {
	nodeRadius = radius;
	nodeBrush = QBrush(fillColor);
    nodePen = QPen(lineColor);
    nodePen.setWidthF(lineWidth);
}

void RenderWidget::setEdgeProperties(QColor lineColor, double lineWidth) {
	edgePen = QPen(lineColor);
	edgePen.setWidthF(lineWidth);
}

void RenderWidget::paintEvent(QPaintEvent*  event) {
    QPainter painter(this);
	painter.fillRect(rect(), backgroundSolid);

    if (pixmap.isNull() || clear) {
        painter.setPen(Qt::white);
        painter.drawText(rect(), Qt::AlignCenter, tr("Rendering graph image, please wait..."));
		pixmap = QPixmap();
		clear = false;
        return;
    }
	
	if(visibleBackgroundImage)
		painter.drawPixmap(rect(), backgroundImage);

	painter.drawPixmap(rect(), pixmap);

    /*QString text = tr("Use mouse wheel or the '+' and '-' keys to zoom. "
                      "Press and hold left mouse button to scroll.");
    QFontMetrics metrics = painter.fontMetrics();
    int textWidth = metrics.width(text);

    painter.setPen(Qt::NoPen);
    painter.setBrush(QColor(0, 0, 0, 127));
    painter.drawRect((width() - textWidth) / 2 - 5, 0, textWidth + 10, metrics.lineSpacing() + 5);
    painter.setPen(Qt::white);
    painter.drawText((width() - textWidth) / 2, metrics.leading() + metrics.ascent(), text);*/
}

void RenderWidget::resizeEvent(QResizeEvent * event) {
   //take care of lag while drawing big things
	QSize old = event->oldSize();
	QSize now = event->size();
	if(visibleBackgroundImage && ((qAbs(old.width() - now.width()) > 100 || qAbs(old.height() - now.height()) > 100))) {
		useBackgroundImage(backgroundImagePath);
	}
	refresh();
}

void RenderWidget::refresh(void) {
	thread.render(graph, size(), edgePen, nodeBrush, nodePen, nodeRadius, visibleEdges, visibleNodes, visibleDescriptions, wrap, bundle);	
}

void RenderWidget::updatePixmap(const QImage &image) {
    pixmap = QPixmap::fromImage(image);
    update();
}