#ifndef RENDERWIDGET_H
#define RENDERWIDGET_H

#include <QPixmap>
#include <QWidget>
#include "renderthread.h"

/*! Renderwidget is a QWidget containing all the necessary UI properties for starting the render thread and displaying the render thread's results in the application window. */
class RenderWidget : public QWidget {
    Q_OBJECT

public:
	/*! Renderwidget constructor.
	 *\param parent pointer to the application window
	 *\param g pointer to the graph
	 */
    RenderWidget(QWidget *parent, Graph* g);
	/*! Sets the background image to a new image specified by the image path. Implementation allows for .png, .jpg and .svg. \param path Path to the new background image. */
	void useBackgroundImage(QString path);
	void useBackgroundColor(QColor color);
	
	bool	getBackgroundActive(void) {return visibleBackgroundImage;};
	bool	getNodesVisible(void) {return visibleNodes;};
	bool	getEdgesVisible(void) {return visibleEdges;};
	bool	getDescriptionsVisible(void) {return visibleDescriptions;};
	bool	getWrap(void) {return wrap;};
	void	refresh(void);
	void	clearWidget() {clear = true; repaint();};
protected:
	/*! Reimplementation of Qt's UI paint event. */
    void paintEvent(QPaintEvent *event);
	/*! Reimplementation of Qt's UI resize event. */
    void resizeEvent(QResizeEvent *event);

public slots:
	void	useBackground(bool set) {visibleBackgroundImage = set; repaint();};
	void	drawNodes(bool draw) {visibleNodes = draw; refresh();};
	void	drawEdges(bool draw) {visibleEdges = draw; refresh();};
	void	showDescriptions(bool show) {visibleDescriptions = show; refresh();};
	void	wrapEdges(bool w) {wrap = w; refresh();};
	void	bundleEdges(bool b) {bundle = b;};
	
	/*! Setter for the visual properties of the graph's nodes \param radius nodes' radius \param fillColor color the node circle is filled with \param lineColor color of the nodes' outlines \param lineWidth width of the nodes' outlines */
	void setNodeProperties(double radius, QColor fillColor, QColor lineColor, double lineWidth);
	/*! Setter for the visual properties of the graph's edges \param lineColor color of the edges' lines \param lineWidth width of the edges' lines */
	void setEdgeProperties(QColor lineColor, double lineWidth);

protected slots:
	/*! Slot that is called from the RenderThread when it has finished the render process and is ready to deliver a result image. \param image the updated rendered image */
    void updatePixmap(const QImage &image);

protected:
	/*  BACKGROUND */
	QString	backgroundImagePath;
	bool	visibleBackgroundImage;
	QPixmap	backgroundImage;
	QBrush	backgroundSolid;

	/* NODE PROPERTIES */
	bool visibleNodes;
	QBrush nodeBrush;
	QPen nodePen;
	double nodeRadius;
	bool visibleDescriptions;
	
	/* EDGE PROPERTIES */
	bool visibleEdges;
	QPen edgePen;
	bool wrap;
	bool bundle;
		
	bool clear;
	Graph* graph;
    RenderThread thread;
    QPixmap pixmap;
};

#endif // RENDERWIDGET_H