#define _USE_MATH_DEFINES
#include <math.h>
#include "visualisierung.h"
#include "flightdata.h"
#include "marveldata.h"
#include "miserablesdata.h"
#include <QTimer>
#include <QGroupBox>
#include <QDebug>
#include <QLabel>

/*!
* Constructor for the application's main window
*/
Visualisierung::Visualisierung() {
	mainWidget = NULL;
	renderWidget = NULL;
	toolBoxWidget = NULL;
	wrapCheckBox = NULL;
	backgroundCheckBox = NULL;
	nodesCheckBox = NULL;
	edgesCheckBox = NULL;
	datasetComboBox = NULL;

	cycles = 6;
	iterations = 10;
	springConstant = 5;
	bundle = false;

	setWindowTitle(tr("Visualization: Edge Bundling"));
	resize(1024,768);

	initInterface();
	useDataSet("Les Misrables");
	connect(&forceBundler, SIGNAL(setProgress(int)), this, SLOT(updateProgress(int)));
	
	//useDataSet("Flights");
	//bundle(0.01f, true);
	//doFDEB(1, 1.0f);	//added Sebi
}

/*!
* Destructor for the application's main window
*/
Visualisierung::~Visualisierung() {}

/*!
* Initializes the application's user interface components.
*/
void Visualisierung::initInterface(void) {
	mainWidget = new QWidget(this);
	setCentralWidget(mainWidget);  

	QGridLayout* layout = new QGridLayout(mainWidget);
	layout->setColumnStretch(0, 10);

	renderWidget = new RenderWidget(this, &graph);
	initRenderWidget();
	layout->addWidget(renderWidget, 0, 0);
	
	toolBoxWidget = new QWidget(this);
	layout->addWidget(toolBoxWidget, 0, 1);
	initToolbox();

	progressBar = new QProgressBar();
	this->statusBar()->addWidget(progressBar, 1);
}

/*!
* Initializes the render widget and defines the initial drawing parameters.
*/
void Visualisierung::initRenderWidget() {
	//initialize default values for widget drawing parameters: later specify all that stuff in GUI?
	renderWidget->setNodeProperties(2, QColor(0xa6, 0xce, 0x39), Qt::black, 1);
	renderWidget->setEdgeProperties(QColor(0, 0, 0, 70), 1);
	renderWidget->drawNodes(true);
	renderWidget->drawEdges(true);
	renderWidget->wrapEdges(false);

}

/*!
* Initializes the interface's toolbox items and sets their initial values.
*/
void Visualisierung::initToolbox(void) {
	datasetComboBox = new QComboBox();
	datasetComboBox->addItem("Flights");
	datasetComboBox->addItem("Marvel");
	datasetComboBox->addItem(QStringLiteral("Les Misrables"));
	datasetComboBox->setCurrentIndex(2);
	datasetComboBox->setToolTip("Select dataset");
	connect(datasetComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(useDataSet(QString)));

	wrapCheckBox = new QCheckBox(QString("wrap"), toolBoxWidget);
	wrapCheckBox->setChecked(renderWidget->getWrap());
	wrapCheckBox->setVisible(wrapPossible);
	wrapCheckBox->setToolTip("Allow edges to be drawn across the horizontal image borders");
	connect(wrapCheckBox, SIGNAL(toggled(bool)), renderWidget, SLOT(wrapEdges(bool)));
	
	backgroundCheckBox = new QCheckBox(QString("Background"), toolBoxWidget);
	backgroundCheckBox->setChecked(renderWidget->getBackgroundActive());
	backgroundCheckBox->setVisible(hasBackgroundImage);
	backgroundCheckBox->setToolTip("Show background image");
	connect(backgroundCheckBox, SIGNAL(toggled(bool)), renderWidget, SLOT(useBackground(bool)));

	nodesCheckBox = new QCheckBox(QString("visible"), toolBoxWidget);
	nodesCheckBox->setChecked(renderWidget->getNodesVisible());
	nodesCheckBox->setToolTip("Display nodes in graph");
	connect(nodesCheckBox, SIGNAL(toggled(bool)), renderWidget, SLOT(drawNodes(bool)));

	descriptionsCheckBox = new QCheckBox(QString("show description"), toolBoxWidget);
	descriptionsCheckBox->setChecked(renderWidget->getDescriptionsVisible());
	nodesCheckBox->setToolTip("Show node descriptions.");
	connect(descriptionsCheckBox, SIGNAL(toggled(bool)), renderWidget, SLOT(showDescriptions(bool)));

	edgesCheckBox = new QCheckBox(QString("visible"), toolBoxWidget);
	edgesCheckBox->setChecked(renderWidget->getEdgesVisible());
	edgesCheckBox->setToolTip("Display edges in graph");
	connect(edgesCheckBox, SIGNAL(toggled(bool)), renderWidget, SLOT(drawEdges(bool)));

	//bundleCheckBox = new QCheckBox(QString("bundle"), toolBoxWidget);
	//bundleCheckBox->setChecked(bundle);
	//bundleCheckBox->setToolTip("Bundle edges");
	//connect(bundleCheckBox, SIGNAL(toggled(bool)), this, SLOT(bundleEdges(bool)));
	//connect(bundleCheckBox, SIGNAL(toggled(bool)), renderWidget, SLOT(bundleEdges(bool)));

	cyclesSpinBox = new QSpinBox(toolBoxWidget);
	cyclesSpinBox->setToolTip("Number of cycles for bundling");
	cyclesSpinBox->setRange(1,10);
	cyclesSpinBox->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
	cyclesSpinBox->stepBy(1);
	cyclesSpinBox->setValue(cycles);
	cyclesSpinBox->setKeyboardTracking(false);
	connect(cyclesSpinBox, SIGNAL(valueChanged(int)), this, SLOT(setCycles(int)));

	iterationsSpinBox = new QSpinBox(toolBoxWidget);
	iterationsSpinBox->setToolTip("Number of iterations for bundling");
	iterationsSpinBox->setRange(1,50);
	iterationsSpinBox->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
	iterationsSpinBox->stepBy(1);
	iterationsSpinBox->setValue(iterations);
	iterationsSpinBox->setKeyboardTracking(false);
	connect(iterationsSpinBox, SIGNAL(valueChanged(int)), this, SLOT(setIterations(int)));

	springSpinBox = new QDoubleSpinBox(toolBoxWidget);
	springSpinBox->setToolTip("Spring force for edge bundling");
	springSpinBox->setRange(0,20);
	springSpinBox->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
	springSpinBox->stepBy(0.1);
	springSpinBox->setValue(springConstant);
	springSpinBox->setKeyboardTracking(false);
	connect(springSpinBox, SIGNAL(valueChanged(double)), this, SLOT(setSpringConstant(double)));

	bundleButton = new QPushButton("Bundle edges", toolBoxWidget);
	connect(bundleButton, SIGNAL(clicked()), this, SLOT(bundleEdges()));
	
	/* DATA TOOLBOX ITEMS */
	QGroupBox *groupBox_data = new QGroupBox(tr("Data"));
	QVBoxLayout *vBox_data = new QVBoxLayout(groupBox_data);
    vBox_data->addWidget(datasetComboBox);
	vBox_data->addWidget(backgroundCheckBox);

	/* EDGES TOOLBOX ITEMS */
	QGroupBox *groupBox_edges = new QGroupBox(tr("Edges"));
	QVBoxLayout *vBox_edges = new QVBoxLayout(groupBox_edges);
    vBox_edges->addWidget(edgesCheckBox);
	vBox_edges->addWidget(wrapCheckBox);
	//vBox_edges->addWidget(bundleCheckBox);
	
	vBox_edges->addWidget(new QLabel("Cycles"));
	vBox_edges->addWidget(cyclesSpinBox);

	vBox_edges->addWidget(new QLabel("Iterations"));
	vBox_edges->addWidget(iterationsSpinBox);

	vBox_edges->addWidget(new QLabel("Spring Force"));
	vBox_edges->addWidget(springSpinBox);
	
	vBox_edges->addWidget(bundleButton);

	/* NODES TOOLBOX ITEMS */
	QGroupBox *groupBox_nodes = new QGroupBox(tr("Nodes"));
	QVBoxLayout *vBox_nodes = new QVBoxLayout(groupBox_nodes);
    vBox_nodes->addWidget(nodesCheckBox);
	vBox_nodes->addWidget(descriptionsCheckBox);

	/* TOOLBOX LAYOUT */
	QVBoxLayout* toolBoxLayout = new QVBoxLayout(toolBoxWidget);
	toolBoxLayout->setAlignment(Qt::AlignTop);
	toolBoxLayout->addWidget(groupBox_data);
	toolBoxLayout->addWidget(groupBox_nodes);
	toolBoxLayout->addWidget(groupBox_edges);
}

/*!
* An update for the toolbox interface is called whenever some graph properties are changed and the toolbox needs to be refreshed accordingly.
*/
void Visualisierung::updateToolbox(void) {
	if(wrapCheckBox)		wrapCheckBox->setVisible(wrapPossible);
	if(backgroundCheckBox)	backgroundCheckBox->setVisible(hasBackgroundImage);
	if(wrapCheckBox)		wrapCheckBox->setChecked(renderWidget->getWrap());
	if(backgroundCheckBox)	backgroundCheckBox->setChecked(renderWidget->getBackgroundActive());
	if(nodesCheckBox)		nodesCheckBox->setChecked(renderWidget->getNodesVisible());
	if(edgesCheckBox)		edgesCheckBox->setChecked(renderWidget->getEdgesVisible());
	if(bundleButton)		bundleButton->setText(bundle?"Unbundle":"Bundle");
}
/* SLOTS for signals and slots */ 
/*!
* useDataset is called whenever a new dataset is selected from the corresponding ComboBox.
* \param name is the dataset's name specified as a string.
*/
void Visualisierung::useDataSet(QString name) {
	graph.clear();
	bundle = false;
	
	Data* dataset;
	if(name == "Flights") {
		dataset = new FlightData(&graph, "Resources/flights/airports_reduced.dat", "Resources/flights/routes_reduced.dat");
		hasBackgroundImage = true;
		wrapPossible = true;
		renderWidget->useBackgroundImage("Resources/flights/mercator_projection.svg");
		} else if(name == "Marvel") {
		dataset = new MarvelData(&graph, "Resources/marvel/names_reduced.dat", "Resources/marvel/edges_reduced.dat");
		hasBackgroundImage = false;
		wrapPossible = false;
		renderWidget->useBackgroundColor(Qt::white);
	} else {
		dataset = new MiserablesData(&graph, "Resources/lesmiserables/names.dat", "Resources/lesmiserables/connections.dat", "Resources/lesmiserables/groups.dat");		
		hasBackgroundImage = false;
		wrapPossible = false;
		renderWidget->useBackgroundColor(Qt::white);
	} 
	renderWidget->wrapEdges(wrapPossible);
	renderWidget->bundleEdges(bundle);
	renderWidget->clearWidget();
	connect(dataset, SIGNAL(setStatus(QString)), this, SLOT(updateStatus(QString)));
	dataset->initGraph();
	forceBundler.setGraph(&graph);
	updateToolbox();
	delete dataset;

	renderWidget->refresh();
}

/*!
* The bundle edges slot is called when the bundle button in the user interface is pressed or when some other function calls this slot.
*/
void Visualisierung::bundleEdges() {
	bundle = !bundle;
	updateToolbox();
	if(bundle) forceBundler.bundleEdges(cycles, iterations, springConstant);
	 
	renderWidget->bundleEdges(bundle);
	renderWidget->refresh();
}